/*
 * Copyright (c) 2005, John Mettraux, OpenWFE.org
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 * . Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.  
 * 
 * . Redistributions in binary form must reproduce the above copyright notice, 
 *   this list of conditions and the following disclaimer in the documentation 
 *   and/or other materials provided with the distribution.
 * 
 * . Neither the name of the "OpenWFE" nor the names of its contributors may be
 *   used to endorse or promote products derived from this software without
 *   specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: OrQueryItem.java 1882 2005-05-17 16:41:07Z jmettraux $
 */

//
// OrQueryItem.java
//
// john.mettraux@openwfe.org
//
// generated with 
// jtmpl 1.1.01 2004/05/19 (john.mettraux@openwfe.org)
//

package openwfe.org.query.item;

import openwfe.org.query.QueryException;
import openwfe.org.query.QueryConfiguration;
import openwfe.org.query.sets.Set;
import openwfe.org.query.sets.SetUtils;


/**
 * A basic implementation of an 'or' (union).
 *
 * <p><font size=2>CVS Info :
 * <br>$Author: jmettraux $
 * <br>$Id: OrQueryItem.java 1882 2005-05-17 16:41:07Z jmettraux $ </font>
 *
 * @author john.mettraux@openwfe.org
 */
public abstract class OrQueryItem

    extends CompositeQueryItem

{

    /*
    private final static org.apache.log4j.Logger log = org.apache.log4j.Logger
        .getLogger(OrQueryItem.class.getName());
    */

    //
    // CONSTANTS & co

    //
    // FIELDS

    //
    // CONSTRUCTORS

    //
    // METHODS from QueryItem

    public Set query (final Object o)
        throws QueryException
    {
        if (getChildren() == null || getChildren().size() < 1) return null;

        Set result = null;

        final java.util.Iterator it = getChildren().iterator();
        while (it.hasNext())
        {
            final QueryItem qi = (QueryItem)it.next();

            if (result == null)
                result = qi.query(o);
            else
                result = SetUtils.doUnion(result, qi.query(o));
        }

        return result;
    }

    public boolean validate (final Object o)
        throws QueryException
    {
        //log.debug("validate()");

        if (getChildren() == null || getChildren().size() < 1) 
        {
            //log.debug("validate() no children : returning false");
            return false;
        }

        final java.util.Iterator it = getChildren().iterator();
        while (it.hasNext())
        {
            final QueryItem qi = (QueryItem)it.next();

            if (qi.validate(o)) return true;
        }

        //log.debug("validate() finally returning false");

        return false;
    }

    public String toString ()
    {
        final StringBuffer sb = new StringBuffer();

        sb.append("<");
        sb.append(this.getClass().getName());
        sb.append("::or>\n");

        final java.util.Iterator it = getChildren().iterator();
        while (it.hasNext())
        {
            final QueryItem qi = (QueryItem)it.next();
            sb.append(qi.toString());
        }

        sb.append("</");
        sb.append(this.getClass().getName());
        sb.append("::or>\n");

        return sb.toString();
    }

    //
    // METHODS

    //
    // STATIC METHODS

}
